@@ -158,10 +158,10 @@ _rfb_authenticate()
 	 /* we didnt get a password on the command line, so go get one */
          if (!opt.password) opt.password = getpass("Password: ");
 
-	 if (!read_from_rfb_server(sock, challenge_and_response, CHALLENGESIZE))
+	 if (!read_from_rfb_server(sock, (char *)challenge_and_response, CHALLENGESIZE))
 	    return 0;
 	 vncEncryptBytes(challenge_and_response, opt.password);
-	 if (!write_exact(sock, challenge_and_response, CHALLENGESIZE))
+	 if (!write_exact(sock, (char *)challenge_and_response, CHALLENGESIZE))
 	    return 0;
 	 if (!read_from_rfb_server(sock, (char*)&auth_result, 4))
 	    return 0;
@@ -250,7 +250,7 @@ rfb_set_format_and_encodings()
    if (!write_exact(sock, (char*)&pf, sz_rfbSetPixelFormatMsg)) return 0;
 
    em.type = rfbSetEncodings;
-   em.nEncodings = 0;
+   em.nEncodings = Swap16IfLE(0);
 
    /* figure out the encodings string given on the command line */
    next = strtok(opt.encodings, " ");
@@ -712,10 +712,10 @@ rfb_get_rgb_from_data(int *r, int *g, int *b, char *data)
  	 *b = (( foo16 >> opt.client.blueshift ) & opt.client.bluemax)<<3;
 	 break;
       case 24:
-	 printf("FIXME unimplemented\n");
-	 break;
       case 32:
-	 printf("FIXME unimplemented\n");
+         *r=data[2]&0x00FF;
+	 *g=data[1]&0x00FF;
+	 *b=data[0]&0x00FF;
 	 break;
    }
 }
